#include "sim.h"

int WMem(int value, unsigned int locn, int size)
     /* Writepoint is hit AFTER write is completed */
{
 if (locn >= MEM_SIZE)
 {
  trapNo= NO_MEM_TRAP | MEM_WRITE_FLAG;
  trapAddr= locn;
  return 1;  /* Failure */
 }
 else if (size == 1)
 {
  if (!ignoreBpt && numBpts[WRITE_BPT] && chkBpt(locn, WRITE_BPT, size))
  {
   trapNo= WBPT_TRAP;
   return 2;
  }
  mem[locn]= value;
  return 0;  /* Success */
 }
 else if (size == 2)
  if ((locn & 1) != 0)
  {
   trapNo= MISALIGNED_TRAP | MEM_WRITE_FLAG;
   trapAddr= locn;
   return 1;
  }
  else if (!ignoreBpt && numBpts[WRITE_BPT] && chkBpt(locn, WRITE_BPT, size))
  {
   trapNo= WBPT_TRAP;
   return 2;
  }
  else
  {
   smem(locn)= (unsigned short)value;
   return 0;
  }
 else if (size == 4)
  if ((locn & 3) != 0)
  {
   trapNo= MISALIGNED_TRAP | MEM_WRITE_FLAG;
   trapAddr= locn;
   return 1;
  }
  else if (!ignoreBpt && numBpts[WRITE_BPT] && chkBpt(locn, WRITE_BPT, size))
  {
   trapNo= WBPT_TRAP;
   return 2;
  }
  else
  {
   imem(locn)= (unsigned int)value;
   return 0;
  }
 else
 {
  message("PANIC: Unknown memory unit size %d in WMem()", size);
  trapNo= PANIC_TRAP;
  return 1;
 }
}

int FetchMem(unsigned int *value, unsigned int locn)
     /* Breakpoint is hit BEFORE instruction is fetched */
{
 if (locn >= MEM_SIZE)
 {
  trapNo= NO_MEM_TRAP | MEM_FETCH_FLAG;
  trapAddr= locn;
  return 1;  /* Failure */
 }
 else if ((locn & 3) != 0)
 {
  trapNo= MISALIGNED_TRAP | MEM_FETCH_FLAG;
  trapAddr= locn;
  return 1;
 }
 else if (!ignoreBpt && numBpts[FETCH_BPT] && chkBpt(locn, FETCH_BPT, 4))
 {
  trapNo= IBPT_TRAP;
  return 2;  /* Stop instruction */
 }
 else
 {
  *value= (unsigned int)imem(locn);
  return 0;
 }
}

int RMem(unsigned int *value, unsigned int locn, int size)
     /* Readpoint is hit AFTER read is completed */
{
 if (locn >= MEM_SIZE)
 {
  trapNo= NO_MEM_TRAP | MEM_READ_FLAG;
  trapAddr= locn;
  return 1;  /* Failure */
 }
 else if (size == 1)
 {
  if (!ignoreBpt && numBpts[READ_BPT] && chkBpt(locn, READ_BPT, size))
  {
   trapNo= RBPT_TRAP;
   return 2;
  }
  *value= (unsigned int)mem[locn];
  return 0;  /* Success */
 }
 else if (size == 2)
  if ((locn & 1) != 0)
  {
   trapNo= MISALIGNED_TRAP | MEM_READ_FLAG;
   trapAddr= locn;
   return 1;
  }
  else
  {
   if (!ignoreBpt && numBpts[READ_BPT] && chkBpt(locn, READ_BPT, size))
   {
    trapNo= RBPT_TRAP;
    return 2;
   }
   *value= (unsigned int)smem(locn);
   return 0;
  }
 else if (size == 4)
  if ((locn & 3) != 0)
  {
   trapNo= MISALIGNED_TRAP | MEM_READ_FLAG;
   trapAddr= locn;
   return 1;
  }
  else
  {
   if (!ignoreBpt && numBpts[READ_BPT] && chkBpt(locn, READ_BPT, size))
   {
    trapNo= RBPT_TRAP;
    return 2;
   }
   *value= (unsigned int)imem(locn);
   return 0;
  }
 else
 {
  message("PANIC:  Unknown memory unit size %d in RMem()", size);
  trapNo= PANIC_TRAP;
  return 1;
 }
}
